#!/bin/sh

# Parameters:
# $1 ... JAVA_HOME
# $2 ... LANG
# $3 ... PID_FILE
# $4 ... MINIM_STDIN_PIPE
# $5 ... APP_DATA
# $6 ... SYNOPKG_PKGDEST
# $7 ... MINIM_HOME
JAVA_HOME="$1"
LANG_VAL="$2"
PID_FILE="$3"
MINIM_STDIN_PIPE="$4"
APP_DATA="$5"
SYNOPKG_PKGDEST="$6"
MINIM_HOME="$7"

# initialization
MINIM_OPTS="--noprompt --nohup"
PID_FILE_DIR=$(dirname "$PID_FILE")
MINIM_STDIN_PIPE_DIR=$(dirname "$MINIM_STDIN_PIPE")

[ ! -e "$MINIM_STDIN_PIPE_DIR" ] && mkdir "$MINIM_STDIN_PIPE_DIR"
[ -e "$MINIM_STDIN_PIPE" ] && rm "$MINIM_STDIN_PIPE"
mknod -m a=rw "$MINIM_STDIN_PIPE" p
[ ! -d "$MINIM_HOME/$APP_DATA" ] && mkdir "$MINIM_HOME/$APP_DATA"
[ ! -e "$PID_FILE_DIR" ] && mkdir "$PID_FILE_DIR"

# run MinimServer and write the pid of the Java process to PID_FILE
[ -n "$LANG_VAL" ] && export LANG="$LANG_VAL"
cd "$MINIM_HOME/$APP_DATA"
tail -n 1 -f "$MINIM_STDIN_PIPE" | LD_LIBRARY_PATH="$SYNOPKG_PKGDEST/libsys" MINIM_LANG="$SYNOPKG_DSM_LANGUAGE" "$JAVA_HOME/bin/java" -jar "$SYNOPKG_PKGDEST/lib/mserver.jar" $MINIM_OPTS > "$MINIM_HOME/$APP_DATA/minimserver-out.log" 2>&1 & echo $! > "$PID_FILE"

# check for a successful MinimServer start
OUT_FIRST_LINE=
COUNTER=15
while [ -z "$OUT_FIRST_LINE" -a "$COUNTER" -ne "0" ]
do
	OUT_FIRST_LINE=$(head -n 1 "$MINIM_HOME/$APP_DATA/minimserver-out.log")
	let COUNTER=$COUNTER-1
	sleep 1
done
if [ -z "$(echo $OUT_FIRST_LINE | grep '^MinimServer.*All rights reserved\.$')" ]; then
    synologset1 sys err 0x11800000 "MinimServer start error: $OUT_FIRST_LINE"
    exit 1
fi

exit 0
