#!/bin/sh
##########################################
#                                        #
# MinimServer Synology package           #
#                                        #
##########################################

PACKAGE_VER="0.8.2"
APP_DATA="appData"
SERVICETOOL="/usr/syno/bin/servicetool"
PACKAGE_LOCATION="/var/packages/${SYNOPKG_PKGNAME}/target"
PREUPGRADE_FLAG="/var/run/minimserver.preupgrade"
appstore_target=$(readlink "${PACKAGE_LOCATION}")
if [ ! -z "${appstore_target}" ]; then
	appstore_dir=$(dirname "${appstore_target}")
	VOLUME=$(dirname "${appstore_dir}")
else
	VOLUME=$(/usr/syno/bin/servicetool --get-alive-volume)
fi
MINIM_HOME="${VOLUME}/${SYNOPKG_PKGNAME}"

preinst ()
{
	# clear flag set by preupgrade
	rm "${PREUPGRADE_FLAG}"

	# check for a proper Java installation, a DSM update may have reset /etc/profile to defaults
	source /etc/profile
	if [ -z "$JAVA_HOME" ]; then
		# search for the missing Java path
		counter=1
		while [ -d "/volume${counter}" ];
		do
			JAVA_HOME="/volume${counter}/@appstore/java6/jre"
			if [ -d "$JAVA_HOME" ]; then break
			fi
			JAVA_HOME="/volume${counter}/@appstore/java7/jre"
			if [ -d "$JAVA_HOME" ]; then break
			fi

			let counter=counter+1
			JAVA_HOME=""
		done
	fi
	if [ -z "$JAVA_HOME" -o ! -f "$JAVA_HOME/bin/java" ]; then
		# Java installation couldn't be detected
		case $SYNOPKG_DSM_LANGUAGE in
			ger)
				echo "Java ist nicht installiert oder nicht richtig konfiguriert. \
				Installieren Sie das Java Synology Paket."
			;;
			*)
				echo "Java is not installed or not properly configured. \
				Download and install the Java Synology package."
			;;
		esac
		exit 1
	fi

	exit 0
}

##################################################################################

postinst ()
{
	# create home directory if it doesn't exist
	if [ ! -d "${MINIM_HOME}" ]; then
		synoshare --add "${SYNOPKG_PKGNAME}" "${SYNOPKG_PKGNAME} configuration" "${MINIM_HOME}" "" "@administrators" "@users" 0 0
		synoacltool -add "${MINIM_HOME}" everyone:*:allow:r-x---a-R-c--:-d-- >/dev/null
		synoacltool -add "${MINIM_HOME}" everyone:*:allow:r-----a-R-c--:f--- >/dev/null
	fi
	if [ ! -d "$MINIM_HOME/$APP_DATA" ]; then
		mkdir "$MINIM_HOME/$APP_DATA"
	fi

	# restore the backup config file if it exists
	if [ "${SYNOPKG_PKG_STATUS}" == "INSTALL" -a -f "$MINIM_HOME/backup/minimserver.config" ]; then
		backup_config_dir="$MINIM_HOME/backup"
	else
		public_dir=$(cat /usr/syno/etc/smb.conf | sed -r '/\/'public'$/!d;s/^.*path=(\/volume[0-9]{1,4}\/'public').*$/\1/')
		if [ "${SYNOPKG_PKG_STATUS}" == "INSTALL" -a -n "$public_dir" -a -f "${public_dir}/minimserver/minimserver.config" ]; then
			backup_config_dir="${public_dir}/minimserver"
		fi
	fi
	if [ -n "${backup_config_dir}" ]; then
		# there should not be a config file in appData, but check for this to be safe
		if [ -f "$MINIM_HOME/$APP_DATA/minimserver.config" ]; then
			rm "$MINIM_HOME/$APP_DATA/minimserver.config"
		fi
		# use the backup config file
		mv "${backup_config_dir}/minimserver.config" "$MINIM_HOME/$APP_DATA"
		# remove the configuration backup directory
		rm -r "${backup_config_dir}"
	fi

	# install the service configure file
	${SERVICETOOL} --install-configure-file --package ${PACKAGE_LOCATION}/etc/MinimServer.sc

	exit 0
}

##################################################################################

preuninst ()
{
	# if upgrading, make a backup of MinimServer application data and extension packages
	if [ "${SYNOPKG_PKG_STATUS}" == "UPGRADE" ]; then
		if [ ! -d "${VOLUME}/@tmp/minimtmp" ]; then
			mkdir ${VOLUME}/@tmp/minimtmp
		fi
		if [ -d "${MINIM_HOME}/${APP_DATA}" ]; then
			# if backup already done by preupgrade, replace it
			if [ -d "${VOLUME}/@tmp/minimtmp/${APP_DATA}" ]; then
				rm -r "${VOLUME}/@tmp/minimtmp/${APP_DATA}"
			fi
			cp -pr ${MINIM_HOME}/${APP_DATA} ${VOLUME}/@tmp/minimtmp
		fi
		if [ -d "${PACKAGE_LOCATION}/libext" ]; then
			# if backup already done by preupgrade, replace it
			if [ -d "${VOLUME}/@tmp/minimtmp/libext" ]; then
				rm -r "${VOLUME}/@tmp/minimtmp/libext"
			fi
			cp -pr ${PACKAGE_LOCATION}/libext ${VOLUME}/@tmp/minimtmp
		fi
	fi

	# uninstall the service configure file
	if [ "${SYNOPKG_PKG_STATUS}" == "UNINSTALL" -o ! -f "${PREUPGRADE_FLAG}" ]; then
		# only remove service configure file when uninstalling the package or downgrading
		${SERVICETOOL} --remove-configure-file --package MinimServer.sc
	fi

	# backup minimserver.config if chosen in the uninstallation dialog
	public_dir=$(cat /usr/syno/etc/smb.conf | sed -r '/\/'public'$/!d;s/^.*path=(\/volume[0-9]{1,4}\/'public').*$/\1/')
	if [ -n "${public_dir}" ]; then
		backup_dir="${public_dir}/minimserver"
	else
		backup_dir="${MINIM_HOME}/backup"
	fi
	if [ "$backup_key" == "true" ]; then
		# copy the configuration file to the backup directory
		[ ! -d "${backup_dir}" ] && mkdir "${backup_dir}"
		cp -p "${MINIM_HOME}/${APP_DATA}/minimserver.config" "${backup_dir}"
	elif [ "$backup_key" == "false" ]; then
		# remove the configuration backup directory if it exists
		if [ -d "${backup_dir}" ]; then
			rm -r "${backup_dir}"
		fi
	fi

	# remove appData directory
	if [ -d "${MINIM_HOME}/${APP_DATA}" ]; then
		rm -r "${MINIM_HOME}/${APP_DATA}"
	fi

	# remove home directory if uninstalling or downgrading
	if [ -f "${PREUPGRADE_FLAG}" ]; then
		preupgrade_ver=$(cat "${PREUPGRADE_FLAG}")
	fi
	if [ "${SYNOPKG_PKG_STATUS}" == "UNINSTALL" -o "${preupgrade_ver}" \< "0.8.1b" ]; then
		# don't remove home directory if it contains a configuration backup or user-created files
		if [ -z "$(ls -A "${MINIM_HOME}")" ]; then
			synoshare --del TRUE "${SYNOPKG_PKGNAME}"
		fi
	fi

	# clear flag set by preupgrade
	rm "${PREUPGRADE_FLAG}"

	exit 0
}

##################################################################################

postuninst ()
{
	exit 0
}

##################################################################################

preupgrade ()
{
	# set flag for use in preuninst
	echo "${PACKAGE_VER}" > "${PREUPGRADE_FLAG}"

	# make a backup of MinimServer application data and extension packages
	# from 0.8.1b onwards, this backup is done by preuninst
	# the backup code here is needed if the package being upgraded is older than 0.8.1b
	OLD_MINIM_HOME=$(cat /etc/passwd | grep "${SYNOPKG_PKGNAME} daemon user" | cut -f6 -d':')
	if [ -d "${OLD_MINIM_HOME}/${APP_DATA}" ]; then
		if [ ! -d "${VOLUME}/@tmp/minimtmp" ]; then
			mkdir ${VOLUME}/@tmp/minimtmp
		fi
		cp -pr ${OLD_MINIM_HOME}/${APP_DATA} ${VOLUME}/@tmp/minimtmp
		if [ -d "${PACKAGE_LOCATION}/libext" ]; then
			cp -pr ${PACKAGE_LOCATION}/libext ${VOLUME}/@tmp/minimtmp
		fi
	fi

	exit 0
}

##################################################################################

postupgrade ()
{
	# restore the backup of MinimServer application data and extensions
	if [ -d "${VOLUME}/@tmp/minimtmp" ]; then
		if [ -d "${VOLUME}/@tmp/minimtmp/${APP_DATA}" ]; then
			cp -pr ${VOLUME}/@tmp/minimtmp/${APP_DATA} ${MINIM_HOME}
			# the files in appData might have been created by the daemon user
			chown -R "root:root" ${MINIM_HOME}/${APP_DATA}
		fi
		if [ -d "${VOLUME}/@tmp/minimtmp/libext" ]; then
			cp -pr ${VOLUME}/@tmp/minimtmp/libext ${PACKAGE_LOCATION}
			# the files in libext might have been created by the daemon user
			chown -R "root:root" ${VOLUME}/@tmp/minimtmp/libext
		fi
		rm -r ${VOLUME}/@tmp/minimtmp
	fi

	exit 0
}
